package net.pricefx.logic.core.config;

import com.fasterxml.jackson.annotation.JsonIgnore;
import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import groovy.lang.MissingPropertyException;
import net.pricefx.logic.core.client.PartitionFactory;
import net.pricefx.restapiclient.client.AbstractPfxAuthV1;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class Config {

	private static final Logger log = LoggerFactory.getLogger(Config.class);

	/** Default config filename. */
	public static final String CONFIG_JSON = "config.json";

	@Deprecated
	public static final String CONFIG_FILENAME = "config.groovy";

	/** Name of this variable config file. */
	public static final String CONNECTIONS_NAME = "CONNECTIONS";

	/** Name of this variable config file. */
	public static final String DEBUG_NAME = "DEBUG";

  /** Name of this variable config file. */
	public static final String KEEP_LABELS_INTACT = "KEEP_LABELS_INTACT";


	@JsonIgnore
	private List<ChangeListener> changeListeners = new ArrayList<>(3);

	/** Value of this variable in config file. */
	Map<String, Map<String, Partition>> envs;

	@JsonIgnore
	private boolean isMigrated;


	/** Value of this variable in config file. */
	private boolean debug;

	/** Value of this variable in config file. */
	private boolean keepLabelsIntact;


	@JsonIgnore
	private boolean isLoaded;


	public Config() {
		init();
	}

	public void reset() {
		init();
		stateChanged(new ChangeEvent(this));
	}

	private void init() {
		envs = new HashMap<>();
		debug = false;
		keepLabelsIntact = false;
		isLoaded = false;
		isMigrated = false;
	}

	public File findConfigFile(Path configDirPath) {

		File dir = configDirPath.toFile();

		do {
			File file = new File(dir, Config.CONFIG_JSON);

			if (file.exists()) {
				isMigrated = true;
				log.info("Found config file " + file);
				return file;
			}

			file = new File(dir, Config.CONFIG_FILENAME);

			if (file.exists()) {
				isMigrated = false;
				log.info("Found config file " + file);
				return file;
			}
			dir = dir.getParentFile();

		} while (dir != null);

		log.info("Config file not found");
		return null;
	}

	@Deprecated
	public synchronized Config load(Path configDir, PartitionFactory partitionFactory) throws IOException {
		init();
		isLoaded = true;

		try {
			if (configDir == null) {
				throw new FileNotFoundException(configDir.toString());
			}

			File file = findConfigFile(configDir);

			if (file == null) {
				throw new FileNotFoundException(configDir.toString());
			}

			log.info("Reading config file '{}'", file.getAbsolutePath());

			if (isMigrated) {
				Partition.getObjectMapper(partitionFactory).readerForUpdating(this).readValue(file);
				initPartitions();
			} else {
				loadGroovy(file, partitionFactory);
			}

			stateChanged(new ChangeEvent(configDir));
			return this;

		} catch (IOException e) {
			stateChanged(new ChangeEvent(configDir));
			throw new IOException(String.format("Cannot find or read config file '%s'", configDir), e);
		}
	}

	@Deprecated
	public synchronized void loadGroovy(File file, PartitionFactory partitionFactory) throws IOException {

		Binding binding = new Binding();
		new GroovyShell(binding).evaluate(file);

		Map<String, Map<String, List<String>>> connectionsMap;

		try {
			connectionsMap = (Map) binding.getProperty(CONNECTIONS_NAME);
		} catch (MissingPropertyException e) {
			String msg = "Specify " + CONNECTIONS_NAME + " in " + CONFIG_FILENAME;
			log.error(msg);
			connectionsMap = new HashMap();
		}

		for (Map.Entry<String, Map<String, List<String>>> conEntry: connectionsMap.entrySet()) {
			assert conEntry.getKey() != null : "Environment definition must contain at least one partition";

			for (Map.Entry<String, List<String>> entry : conEntry.getValue().entrySet()) {
				assert entry.getValue().size() >= 1 : "Invalid partition definition " + entry.getKey() + "@" + entry.getValue()
				+ entry.toString()
				+ " It must contain URL field'";
			}
		}

		// *** Fill Connections bean ***
		from(connectionsMap, partitionFactory);

		// Debug directive
		try {
			debug = (boolean) binding.getProperty(DEBUG_NAME);

		} catch (MissingPropertyException e) {
			debug = false;
			log.debug("Optional directive {} not defined. Using default value '{}'", e.getProperty(), debug);
		}

        // Debug directive
        try {
            keepLabelsIntact = (boolean) binding.getProperty(KEEP_LABELS_INTACT);

        } catch (MissingPropertyException e) {
            keepLabelsIntact = false;
            log.debug("Optional directive {} not defined. Using default value '{}'", e.getProperty(), keepLabelsIntact);
        }
	}


	@Deprecated
	public void from(Map<String, Map<String, List<String>>> map, PartitionFactory partitionFactory) throws IOException {
		envs = new LinkedHashMap<>(map.size());
		for (Map.Entry<String, Map<String, List<String>>> entry : map.entrySet()) {
			Map<String, Partition> partitions = new LinkedHashMap<>();

			for (Map.Entry<String, List<String>> entry2 : entry.getValue().entrySet()) {
				List<String> setting = entry2.getValue();
				String partitionName = entry2.getKey();
				String url = setting.get(0);
				String username = setting.size() >= 2 ? setting.get(1) : null;
				String encryptedPassword = setting.size() >= 3 ? setting.get(2) : null;
				String twoFactorAuthSecurityToken = setting.size() >= 4 ? setting.get(3) : null;
				Partition partition = partitionFactory.createPartition(entry.getKey(), partitionName, url);
				AbstractPfxAuthV1 authenticator = partition.getAuthenticator();
				if (username != null && encryptedPassword != null) {
					authenticator.setCredentials(username, Encryptor.decrypt(encryptedPassword));
				}
				if (username != null && twoFactorAuthSecurityToken != null) {
					authenticator.setTfaToken(twoFactorAuthSecurityToken);
				}
				partitions.put(partitionName, partition);
			}

			envs.put(entry.getKey(), partitions);
		}
	}

	private void putPartition(Partition partition) {
		Map<String, Partition> partitionMap = envs.computeIfAbsent(partition.getParentEnvironment(), key -> new HashMap<>());
		partitionMap.put(partition.getPartition(), partition);
	}

	public void addPartitions(List<Partition> partitions) {
		for (Partition partition : partitions) {
			putPartition(partition);
		}
		stateChanged(new ChangeEvent(this));
	}

	public void addPartition(Partition partition) {
		putPartition(partition);
		stateChanged(new ChangeEvent(this));
	}


	public Map<String, Partition> getPartitions(String env) {
		return envs.get(env);
	}

	private void initPartitions() {
		if (envs != null) {
			for (Map.Entry<String, Map<String, Partition>> envEntry : envs.entrySet()) {
				Map<String, Partition> partitions = envEntry.getValue();
				if (partitions != null) {
					for (Map.Entry<String, Partition> entry : partitions.entrySet()) {
						Partition partition = entry.getValue();
						partition.setParentEnvironment(envEntry.getKey());
					}
				}
			}
		}
	}

	public void rebuildClients() {
		if (envs != null) {
			for (Map<String, Partition> partitions : envs.values()) {
				if (partitions != null) {
					for (Partition partition : partitions.values()) {
						partition.build();
					}
				}
			}
		}
	}


	public Partition findPartition(String env, String partitionName) {
		Map<String, Partition> partitions = envs.get(env);
		if (partitions != null) {
			return partitions.get(partitionName);
		}

		return null;
	}

	public void addChangeListener(ChangeListener listener) {
		changeListeners.add(listener);
	}

	public void removeListener(ChangeListener listener) {
		changeListeners.remove(listener);
	}

	public void removeListeners() {
		changeListeners.clear();
	}

	public void stateChanged(ChangeEvent e) {
		for (ChangeListener listener : changeListeners) {
			listener.stateChanged(e);
		}
	}

	@JsonIgnore
	public String[] getEnvironmentsArray() {
		if (envs == null) {
			return new String[0];
		}
		return envs.keySet().toArray(new String[envs.size()]);
	}

	public Map<String, Map<String, Partition>> getEnvs() {
		return envs;
	}

	public boolean getKeepLabelsIntact() {
		return keepLabelsIntact;
	}

	@JsonIgnore
	public boolean isLoaded() {
		return isLoaded;
	}

	@JsonIgnore
	public boolean isMigrated() {
		return isMigrated;
	}
}