package net.pricefx.server.dto.calculation;

import com.fasterxml.jackson.annotation.JsonIgnore;
import net.pricefx.common.api.CalculationResultType;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * This class is used to generate the definition of a so called controller dashboard portlet (or wizard).
 * It consitsts generally out of two elements: HTML snippets and buttons
 * The buttons can trigger UI actions (e.g. opening a page) or backend actions (calling a backend URL)
 *
 */
public class DashboardController implements Serializable, ComplexCalculationResult {

	private static final long serialVersionUID = -9173069395089693075L;

	private List<ControllerItem> items;
	
	public DashboardController() {
		items = new ArrayList<>();
	}
	
	@Override
	@JsonIgnore
	public String getResultType() {
		return CalculationResultType.DASHBOARDCONTROLLER.name();
	}

	public List<ControllerItem> getItems() {
		return items;
	}

	/**
	 * Adds a button to the controller
	 *
	 * <p>
	 * Example:
	 * <blockquote>
	 * <pre>
	 * def controller = api.newController()
	 * controller.addButton("1. STEP: Go to Live Price Grids", "priceGridPage")
	 * </pre>
	 * </blockquote>
	 *
	 * @param label The label of the button
	 * @param targetPage The name of the target page. Hint: This is also displayed in the URL after the hash symbol
	 */
	public void addButton(String label, String targetPage) {
		addButton(label, targetPage, null);
	}

	/**
	 * Adds a button to the controller
	 *
	 * <p>
	 * Example:
	 * <blockquote>
	 * <pre>
	 * def controller = api.newController()
	 * controller.addButton("2. STEP: Go to LPG ABC", "priceGridPage", "45.PG")
	 * </pre>
	 * </blockquote>
	 *
	 * @param label The label of the button
	 * @param targetPage The name of the target page. Hint: This is also displayed in the URL after the hash symbol
	 * @param targetPageState A extra parameter that is used for deep links. E.g. a typed id to open a specific page
	 *
	 */
	public void addButton(String label, String targetPage, Object targetPageState) {
		addButton(label, targetPage, targetPageState, null);
	}
	
	/**
	 * Adds a button to the controller
	 *
	 * <p>
	 * Example:
	 * <blockquote>
	 * <pre>
	 * def controller = api.newController()
	 * controller.addButton("Open Dashboard", "dashboardPage", null, "FirstDash")
	 * </pre>
	 * </blockquote>
	 *
	 * @param label The label of the button
	 * @param targetPage The name of the target page. Hint: This is also displayed in the URL after the hash symbol
	 * @param targetPageState A extra parameter that is used for deep links. E.g. a typed id to open a specific page
	 * @param componentId Some UI actions require a parameter in this field (e.g. opening a specific dashboard)
	 */
	public void addButton(String label, String targetPage, Object targetPageState, Object componentId) {
		ControllerItem item = new ControllerItem();
		item.label = label;
		item.type = ControllerItemType.BUTTON;
		item.values = new ArrayList<>();
		item.values.add(targetPage);
		item.values.add(targetPageState);
		if(componentId != null) item.values.add(componentId);
		items.add(item);
	}
	
	/**
	 * Performs a backend call in the user's context (i.e. from the browser)
	 *
	 * <p>
	 * Example:
	 * <blockquote>
	 * <pre>
	 * def controller = api.newController()
	 * controller.addBackendCall("Calculate LPG", "/pricegridmanager.calculate/45")
	 * </pre>
	 * </blockquote>
	 *
	 * @param label The label of the button
	 * @param apiUrl The URL to call
	 */
	public void addBackendCall(String label, String apiUrl) {
		addBackendCall(label, apiUrl, null, null, null);
	}
	
	/**
	 * Performs a backend call in the user's context (i.e. from the browser)
	 *
	 * @param label The label of the button
	 * @param apiUrl The URL to call
	 * @param payload The POST data to send along with the request
	 */	
	public void addBackendCall(String label, String apiUrl, String payload) {
		addBackendCall(label, apiUrl, payload, null, null);
	}
	
	/**
	 * Performs a backend call in the user's context (i.e. from the browser)
	 *
	 * <p>
	 * Example:
	 * <blockquote>
	 * <pre>
	 * def controller = api.newController()
	 * controller.addBackendCall("Calculate LPG", "/pricegridmanager.calculate/45", null, "OK", "Not OK")
	 * </pre>
	 * </blockquote>
	 *
	 * @param label The label of the button
	 * @param apiUrl The URL to call
	 * @param payload The POST data to send along with the request
	 * @param successMessage A text that should be displayed in case of a successful response from the backend
	 * @param failMessage A text that should be displayed in case of a error response from the backend
	 */
	public void addBackendCall(String label, String apiUrl, String payload, String successMessage, String failMessage) {
		ControllerItem item = new ControllerItem();
		item.label = label;
		item.type = ControllerItemType.BACKENDACTION;
		item.values = new ArrayList<>();
		item.values.add(apiUrl);
		item.values.add(payload);
		item.values.add(successMessage);
		item.values.add(failMessage);
		items.add(item);
	}	

	/**
	 * Adds a HTML snippet into the controller
	 *
	 <p>
	 * Example:
	 * <blockquote>
	 * <pre>{@code
	 * def controller = api.newController()
	 * controller.addHTML("<h2>Welcome to Price f(x)</h2>")
	 * }</pre>
	 * </blockquote>
	 *
	 * @param html The html snippet
	 */
	public void addHTML(String html) {
		ControllerItem item = new ControllerItem();
		item.type = ControllerItemType.HTML;
		item.values = new ArrayList<>();
		item.values.add(html);
		items.add(item);
	}

	/**
	 * Performs a backend call in the user's context in new window. Useful for downloading Excel, PDF
	 *
	 * @param label The label of the button
	 * @param url The URL to call
	 */
	public void addDownloadButton(String label, String url) {
		addDownloadButton(label, url, null);
	}

	/**
	 * Performs a backend call in the user's context in new window. Useful for downloading Excel, PDF
	 *
	 * @param label The label of the button
	 * @param url The URL to call
	 * @param payload The POST data to send along with the request
	 */
	public void addDownloadButton(String label, String url, String payload) {
		ControllerItem item = new ControllerItem();
		item.label = label;
		item.type = ControllerItemType.DOWNLOADBUTTON;
		item.values = new ArrayList<>();
		item.values.add(url);
		item.values.add(payload);
		items.add(item);
	}

	public enum ControllerItemType {
		HTML, BUTTON, BACKENDACTION, DOWNLOADBUTTON
	}
	
	public class ControllerItem {
		public ControllerItemType type;
		public String label;
		public List<Object> values;
	}
}
